@@ -16,27 +16,42 @@ in the module's attributes `_IvyServer`.
   IvySendMsg, IvySendDieMsg, IvySendDirectMsg
 :group Timers: IvyTimerRepeatAfter, IvyTimerModify, IvyTimerRemove
 
-Copyright (c) 2005-2011 Sebastien Bigaret <sbigaret@users.sourceforge.net>
+Copyright (c) 2005-2016 Sebastien Bigaret <sbigaret@users.sourceforge.net>
 """
+import sys
+# The next line taken from https://pythonhosted.org/six/
+# Copyright (c) 2010-2016 Benjamin Peterson
+PY2 = sys.version_info[0] == 2
 
-_IvyServer=None
+_IvyServer = None
 
-from ivy import IvyApplicationConnected, IvyApplicationDisconnected
-from ivy import IvyRegexpAdded, IvyRegexpRemoved
-from ivy import void_function
+if PY2:
+    from ivy import IvyApplicationConnected, IvyApplicationDisconnected
+    from ivy import IvyRegexpAdded, IvyRegexpRemoved
+    from ivy import void_function
+else:
+    from .ivy import IvyApplicationConnected, IvyApplicationDisconnected
+    from .ivy import IvyRegexpAdded, IvyRegexpRemoved
+    from .ivy import void_function
+
+import signal
 
 def IvyInit(agent_name, ready_msg=None,
             main_loop_type_ignored=0,
             on_cnx_fct=void_function,
             on_die_fct=void_function):
-  """
-  Initializes the module. This method should be called exactly once before any
-  other method is called.
-  """
-  global _IvyServer
-  assert _IvyServer is None
-  from ivy import IvyServer
-  _IvyServer = IvyServer(agent_name, ready_msg, on_cnx_fct, on_die_fct)
+    """
+    Initializes the module. This method should be called exactly once before any
+    other method is called.
+    """
+    global _IvyServer
+    assert _IvyServer is None
+    if PY2:
+        from ivy import IvyServer
+    else:
+        from .ivy import IvyServer
+
+    _IvyServer = IvyServer(agent_name, ready_msg, on_cnx_fct, on_die_fct)
 
 
 def IvyStart(ivybus=None):
@@ -44,7 +59,7 @@ def IvyStart(ivybus=None):
     Starts the Ivy server and fully activate the client.  Please refer to the
     discussion in `IvyMainLoop()` 's documentation.
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     _IvyServer.start(ivybus)
 
 
@@ -57,12 +72,14 @@ def IvyMainLoop():
     fully functional and begins to receive messages as soon as the `IvyStart()`
     method is called.
     """
-    assert _IvyServer != None
-    _IvyServer.server_termination.wait()
+    assert _IvyServer is not None
+    signal.signal(signal.SIGINT, lambda *arg: IvyStop())
+    while not _IvyServer.server_termination.wait(5):
+        pass
 
 
 def IvyStop():
-    assert _IvyServer != None
+    assert _IvyServer is not None
     _IvyServer.stop()
 
 
@@ -74,42 +91,47 @@ def IvyBindMsg(on_msg_fct, regexp):
     :return: an id identifying the binding, that can be used to unregister it
       (see `IvyUnBindMsg()`)
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     return _IvyServer.bind_msg(on_msg_fct, regexp)
 
+
 def IvyBindDirectMsg(on_msg_fct):
     """
     Registers a method that should be called each time someone sends us
     a direct message
 
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     return _IvyServer.bind_direct_msg(on_msg_fct)
 
-def IvyUnBindMsg(id):
+
+def IvyUnBindMsg(binding_id):
     """
     Unregisters a binding
 
-    :param id: the binding's id, as previously returned by `IvyBindMsg()`.
+    :param binding_id: the binding's id, as previously returned by `IvyBindMsg()`.
     :return: the regexp corresponding to the unsubscribed binding
     :except KeyError: if no such subscription can be found
     """
-    assert _IvyServer != None
-    return _IvyServer.unbind_msg(id)
+    assert _IvyServer is not None
+    return _IvyServer.unbind_msg(binding_id)
+
 
 def IvyBindRegexpChange(regexp_change_callback):
-  """
-  """
-  assert _IvyServer != None
-  return _IvyServer.bind_regexp_change(regexp_change_callback)
+    """
+    """
+    assert _IvyServer is not None
+    return _IvyServer.bind_regexp_change(regexp_change_callback)
+
 
 def IvySendMsg(msg):
     """
     Sends a message on the bus
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     return _IvyServer.send_msg(msg)
 
+
 def IvySendDieMsg(client):
     """
     Sends a "die" message to `client`, instructing him to terminate.
@@ -117,7 +139,7 @@ def IvySendDieMsg(client):
     :param client: an `ivy.IvyClient` object,  as returned by
       `IvyGetApplication()`
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     client.send_die_message()
 
 
@@ -135,7 +157,7 @@ def IvySendDirectMsg(client, num_id, msg):
         along with the message.
       - `msg`: the message to send
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     client.send_direct_message(num_id, msg)
 
 
@@ -152,7 +174,7 @@ def IvySendError(client, num_id, error_msg):
         along with the message.
       - `error_msg`: the message to send
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     client.send_error(num_id, error_msg)
 
 
@@ -160,9 +182,10 @@ def IvyGetApplicationList():
     """
     Returns the names of the applications that are currently connected
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     return _IvyServer.get_clients()
 
+
 def IvyGetApplicationMessages(client):
     """
     Returns all subscriptions for that client
@@ -173,6 +196,7 @@ def IvyGetApplicationMessages(client):
     """
     return client.get_regexps()
 
+
 def IvyGetApplication(name):
     """
     Returns the Ivy client registered on the bus under the given name.
@@ -182,10 +206,11 @@ def IvyGetApplication(name):
     
     :returns: an `ivy.IvyClient` object
     """
-    assert _IvyServer != None
+    assert _IvyServer is not None
     clients = _IvyServer.get_client_with_name(name)
     return clients and clients[0] or None
 
+
 def IvyGetApplicationName(client):
     """
     Equivalent to ``client.agent_name``
@@ -195,6 +220,7 @@ def IvyGetApplicationName(client):
     """
     return client.agent_name
 
+
 def IvyGetApplicationHost(client):
     """
     Equivalent to ``client.fqdn``. IP address is stored under ``client.ip``,
@@ -204,9 +230,11 @@ def IvyGetApplicationHost(client):
       `IvyGetApplication()`
     """
     return client.fqdn
-  
 
-_timers={}
+
+_timers = {}
+
+
 def IvyTimerRepeatAfter(count, delay, callback):
     """
     Creates and activates a new timer
@@ -219,12 +247,16 @@ def IvyTimerRepeatAfter(count, delay, callback):
 
     :return: the timer's id
     """
-    assert _IvyServer != None
-    from ivy import IvyTimer
+    assert _IvyServer is not None
+    if PY2:
+        from ivy import IvyTimer
+    else:
+        from ivy import IvyTimer
+
     # The original python API relies on a callback called without any parameter
     cb = lambda timer, callback=callback: callback()
     timer = IvyTimer(_IvyServer, count, delay, cb)
-    _timers[timer.id]=timer
+    _timers[timer.id] = timer
     timer.start()
     return timer.id
 
@@ -248,29 +280,53 @@ def IvyTimerRemove(timer_id):
     
     :param timer_id: id of the timer, as returned by `IvyTimerRepeatAfter`
     """
-    from ivy import IvyTimer
     timer = _timers[timer_id]
     timer.abort = True
     del _timers[timer_id]
 
 
+def IvyBindPong(on_pong_fct):
+    """
+    Registers a method that should be called when we receive a PONG
+    message.  When receiving a PONG message in replying of a PING
+    message we sent (see IvySendPing), this method is called with
+    three arguments:
 
-#TODO  IvyUnBindMsg(id)
+    - the first one is the IvyClient object sorresponding to the agent
+      sending the message;
 
+    - the second one is the time elapsed between the sending of the
+      ping and the receiving of the pong.
 
-"""
-# copy/paste for quick tests w/ ivyprobe
-from ivy.std_api import *
-IvyInit('Test', 'test welcome', 0) 
-IvyStart()
-def onmsgproc(*larg):
-   print larg
+    """
+    assert _IvyServer is not None
+    return _IvyServer.bind_pong(on_pong_fct)
 
-IvyBindMsg(onmsgproc, '(.*)')
-IvyGetApplicationList()
-app=IvyGetApplication('IVYPROBE')
-IvyGetApplicationName(app)
-IvyGetApplicationHost(app)
-IvySendDirectMsg(app, 765, 'glop')
-IvySendDieMsg(app)
-"""
+def IvySetPongCallback(on_pong_fct):
+    """
+    alias for IvyBindPong (IvySetPongCallback is the name used in ivy-c)
+    """
+    return IvyBindPong(on_pong_fct)
+
+def IvySendPing(client):
+    """
+    Sends a PING message to the client.
+    See also: IvyBindPong()
+    """
+    assert _IvyServer is not None
+    client.send_ping()
+
+## copy/paste for quick tests w/ ivyprobe
+# from ivy.std_api import *
+# IvyInit('Test', 'test welcome', 0)
+# IvyStart()
+# def onmsgproc(*larg):
+#    print larg
+#
+# IvyBindMsg(onmsgproc, '(.*)')
+# IvyGetApplicationList()
+# app=IvyGetApplication('IVYPROBE')
+# IvyGetApplicationName(app)
+# IvyGetApplicationHost(app)
+# IvySendDirectMsg(app, 765, 'glop')
+# IvySendDieMsg(app)
